#!/usr/bin/python2.7

#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

from time import time, sleep
import threading
import os
from txosc import async, dispatch, osc
from twisted.internet import reactor

print("Relay\n MultiPass For Freedom\n\n")

'''
Don't forget to add handler for every message from multicast to udp
otherwise they are not retransmit.
With "/*", all messsages do loop.
'''
# List with all message to retransmit
message_send_from_multicast_to_udp = ["/for_all_players", "/reset", "/new_game"]

class UDPReceiver(object):
    def __init__(self, port):
        self.port = port
        self.receiver = dispatch.Receiver()
        self._server_port = reactor.listenUDP(self.port, async.DatagramServerProtocol(self.receiver))
        print("Listening udp on: %s : %s" % (host_local, self.port))
        # quit
        self.receiver.addCallback("/quit", self.quit_handler)
        # handler for all usefull data from a player
        self.receiver.addCallback("/*", r.send_with_multicast_handler)

    def quit_handler(self, message, address):
        print("Quit : Got %s from %s" % (message, address))
        print("Goodbye.")
        #os._exit(0)

class UDPMulticastSender(object):
    def __init__(self, port, host="224.0.0.14"):
        self.port = port
        self.host = host
        self.client = async.DatagramClientProtocol()
        self._client_port = reactor.listenUDP(0, self.client)
        print("Sending multicast on: %s : %s" % (self.host, self.port))

    def send_message(self, message):
        self.client.send(message, (self.host, self.port))

class UDPMulticastReceiver(object):
    def __init__(self, port):
        self.port = port
        self.receiver = dispatch.Receiver()
        self._server_port = reactor.listenMulticast(self.port, async.MulticastDatagramServerProtocol(self.receiver, host_multicast), listenMultiple=True)
        print("Listening multicast on: %s : %s" % (host_multicast, self.port))
        # if "/*" return received message from blender to blender
        for item in message_send_from_multicast_to_udp:
            self.receiver.addCallback(item, r.send_with_udp_handler)

class UDPSender(object):
    def __init__(self, port, host="127.0.0.1"):
        self.port = port
        self.host = host
        self.client = async.DatagramClientProtocol()
        self._client_port = reactor.listenUDP(0, self.client)
        print("Sending multicast on: %s : %s" % (self.host, self.port))

    def send_message(self, message):
        self.client.send(message, (self.host, self.port))

class relay(object):
    def __init__(self, port_multicast, host_multicast, host_local, port_out, port_in):
        self.port_multicast = port_multicast
        self.host_multicast = host_multicast
        self.host_local = host_local
        self.port_out = port_out
        self.port_in = port_in
        self.local_recv = "Nothing"
        self.local_send = "Everything"

    def relay_main(self):
        self.udp_receiver = UDPReceiver(self.port_in)
        self.udp_sender = UDPSender(self.port_out)
        self.multicast_sender = UDPMulticastSender(self.port_multicast)
        self.multicast_reicever = UDPMulticastReceiver(self.port_multicast)
        reactor.run()

    def send_with_multicast_handler(self, message, address):
        # send the same message to multicast
        self.multicast_sender.send_message(message)
        self.local_recv = message

    def send_with_udp_handler(self, message, address):
        # send the same message with udp to blender
        self.udp_sender.send_message(message)
        self.local_send = message

    def print_some(self):
        while True:
            sleep(2)
            print("Local message sended: %s" % (self.local_send))
            print("Local message received: %s" % (self.local_recv))

if __name__ == "__main__":
    host_multicast = "224.0.0.14"
    port_multicast = 18888
    host_local = "127.0.0.1"
    port_out = 8000
    port_in = 9000

    # relay
    r = relay(port_multicast, host_multicast, host_local, port_out, port_in)
    thread1=threading.Thread(target=r.print_some)
    thread1.start()
    r.relay_main()
