#!/usr/bin/python2.7

#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

'''
To use this script with another project, change handler for exemple:
    To receive all message, without sub-messsage:
        self.receiver.addCallback("/*", some_handler)
    or, with touchosc, for example:
        self.receiver.addCallback("/1/*", some_handler)
    and create some_handler
'''

import os
import sys
import threading
from time import time, sleep

from txosc import async, dispatch, osc
from twisted.internet import reactor

import managementDict


class UDPMulticastSender(object):
    '''doc'''
    def __init__(self, port, host):
        '''doc'''
        self.port = port
        self.host = host
        self.client = async.DatagramClientProtocol()
        self._client_port = reactor.listenUDP(0, self.client)
        print("Sending multicast on: %s : %s" % (self.host, self.port))

    def send_message(self, message):
        '''doc'''
        self.client.send(message, (self.host, self.port))


class UDPMulticastReceiver(object):
    '''doc'''
    def __init__(self, port, host):
        '''doc'''
        self.port = port
        self.host = host
        self.receiver = dispatch.Receiver()
        self._server_port = reactor.listenMulticast(self.port,
            async.MulticastDatagramServerProtocol(self.receiver, self.host),
            listenMultiple=True)
        print("Listening multicast on: %s : %s" % (self.host, self.port))
        # handler for all usefull data from a player
        self.receiver.addCallback("/player_data",
                                    server_class.player_data_handler)
        self.receiver.addCallback("/reset", server_class.reset_handler)


class server_main(object):
    '''doc'''
    def __init__(self, port_multicast=18888, host_multicast="224.0.0.14"):
        self.port_multicast = port_multicast
        self.host_multicast = host_multicast

    def game_server(self):
        '''doc'''
        self.gameDict = managementDict.gameManagement()
        print("Game Server\n\nMultiPass For Freedom\n\Press q + Enter to quit")
        # Set instance
        self.multicast_sender = UDPMulticastSender(self.port_multicast,
                                                    self.host_multicast)
        self.multicast_reicever = UDPMulticastReceiver(self.port_multicast,
                                                        self.host_multicast)
        reactor.run()

    def player_data_handler(self, message, address):
        '''
        This function is called every received message from every player.
        So all data are collected here.
        The frequency of this function is
        the frequency of blender * players number
        With 4 players: 60 * 4 = 240 Hz
        '''
        self.gameDict.insert_data_in_dict(message.getValues(), address)

    def send2players(self):
        '''The frequency of dispatched messages must be 60 Hz because
        Blender received at 60 Hz at the best.'''
        while True:
            sleep(0.015)
            msg_for_all_players = self.gameDict.create_msg_for_all_players()
            self.multicast_sender.send_message(msg_for_all_players)

    def delete_disconnected(self):
        '''doc'''
        while True:
            sleep(1)
            # delete disconnected player every second
            self.gameDict.delete_disconnected_player()
            # Print to watch exchange
            msg_for_all_players = self.gameDict.create_msg_for_all_players()
            print("Message sended: %s" % (msg_for_all_players))

    def reset_handler(self, message, address):
        '''doc'''
        print ("Reset ...")
        self.gameDict.reset_data()


def thread_keyboard():
    '''doc'''
    while True:
        c = sys.stdin.read(1)
        if c == 'q':
            print("Exit")
            os._exit(0)

if __name__ == "__main__":
    host_multicast = '224.0.0.14'
    port_multicast = 18888
    # Conversation
    while True:
        print "\nSelect :\n"
        print "0 - Exit\n"
        print\
        "1 - Multicast IP is 224.0.0.11 and Port is 18888: Change IP or Port\n"
        print "2 - Run Game Server\n"
        #selection = input("Enter selection: ")
        selection = 2 # only to test
        if selection == 0:
            os._exit(0)
        elif selection == 1:
            host_multicast = raw_input('Multicast IP ? ')
            port_multicast = input('Multicast Port ? ')
        elif selection == 2:

            # set instance of server_main class
            server_class = server_main(port_multicast, host_multicast)

            thread1 = threading.Thread(target=thread_keyboard)
            thread1.start()

            thread2 = threading.Thread(target=server_class.send2players)
            thread2.start()

            thread3 = threading.Thread(target=server_class.delete_disconnected)
            thread3.start()

            # Run server
            server_class.game_server()

        elif selection >= 0:
            break
